@@ -6,8 +6,9 @@ use App::cpm::Distribution;
 use App::cpm::Job;
 use App::cpm::Logger;
 use Module::Metadata;
+use IO::Handle;
 use version;
-our $VERSION = '0.351';
+our $VERSION = '0.352';
 
 sub new {
     my ($class, %option) = @_;
@@ -30,18 +31,6 @@ sub new {
     $self;
 }
 
-sub clear {
-    my $self = shift;
-    $self->{installed_distributions} = 0;
-    $self->{jobs} = {};
-    $self->{distributions} = {};
-    $self->{_fail_resolve} = {};
-    $self->{_fail_install} = {};
-    $self->{_is_installed} = {};
-    $self->{_artifacts} = {};
-    $self;
-}
-
 sub fail {
     my $self = shift;
     my @fail_resolve = sort keys %{$self->{_fail_resolve}};
@@ -84,12 +73,49 @@ sub register_result {
 
     %{$job} = %{$result}; # XXX
 
+    my $logged = $self->info($job);
     my $method = "_register_@{[$job->{type}]}_result";
     $self->$method($job);
     $self->remove_job($job);
+    $self->_show_progress if $logged && $self->{show_progress};
+
     return 1;
 }
 
+sub info {
+    my ($self, $job) = @_;
+    my $type = $job->type;
+    return if !$App::cpm::Logger::VERBOSE && $type ne "install";
+    my $name = $job->distvname;
+    my ($message, $optional);
+    if ($type eq "resolve") {
+        $message = $job->{package};
+        $message .= " -> $name" . ($job->{ref} ? "\@$job->{ref}" : "") if $job->{ok};
+        $optional = "from $job->{from}" if $job->{ok} and $job->{from};
+    } else {
+        $message = $name;
+        $optional = "using cache" if $type eq "fetch" and $job->{using_cache};
+    }
+    my $elapsed = defined $job->{elapsed} ? sprintf "(%.3fsec) ", $job->{elapsed} : "";
+
+    App::cpm::Logger->log(
+        pid => $job->{pid},
+        type => $type,
+        result => $job->{ok} ? "DONE" : "FAIL",
+        message => "$elapsed$message",
+        optional => $optional,
+    );
+    return 1;
+}
+
+sub _show_progress {
+    my $self = shift;
+    my $all = keys %{$self->{distributions}};
+    my $num = $self->installed_distributions;
+    print STDERR "--- $num/$all ---";
+    STDERR->flush; # this is needed at least with perl <= 5.24
+}
+
 sub remove_job {
     my ($self, $job) = @_;
     delete $self->{jobs}{$job->uid};
